﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Services.Client;

using mana.Data.Azure.Tables;

using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;
using Microsoft.WindowsAzure.ServiceRuntime;

namespace Library
{
    public static class Parameters
    {
        public static void CreateAzureTableModel()
        {
            try
            {
                Internal.CreateAzureTableModel();
            }
            catch (Exception ex)
            {
                throw new Exception("The Azure Table model couldn't be created. " + ex.Message);
            }   
        }

        #region Internal classes

        internal static class Internal
        {
            public static CloudStorageAccount GetCloudStorageAccount()
            {
                return CloudStorageAccount.Parse(GetFromServiceConfiguration("DataConnectionString"));
            }

            public static void CreateAzureTableModel()
            {
                var model = new TableModel(GetCloudStorageAccount());

                var tableNames = GetAzureTableModelTableNames();
                model.TableNames.AddRange(tableNames);
                model.Synchronize();
            }

            public static CloudBlobContainer GetContactsReportsContainer()
            {
                var account = GetCloudStorageAccount();
                var container = account.CreateCloudBlobClient().GetContainerReference("contacts-reports");
                container.CreateIfNotExist();

                return container;
            }

            private static IEnumerable<string> GetAzureTableModelTableNames()
            {
                return GetFromServiceConfiguration("TableModelTablesNames").Split('|');
            }

            private static string GetFromServiceConfiguration(string name)
            {
                return RoleEnvironment.GetConfigurationSettingValue(name);
            }
        }

        #endregion
    }
}
