using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Threading;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Diagnostics;
using Microsoft.WindowsAzure.ServiceRuntime;
using Microsoft.WindowsAzure.StorageClient;

using Library;

using mana.Data.Azure.Queues;

namespace Processes
{
    public class WorkerRole : RoleEntryPoint
    {
        public override void Run()
        {
            Trace.WriteLine("Processes run.", "Information");

            while (true)
            {
                try
                {
                    ProcessContactReportsGenerationQueue();
                }
                catch (Exception)
                {
                    //  register error
                }

                Thread.Sleep(10000);
                Trace.WriteLine("Working...", "Information");
            }
        }

        private ContactReportGenerationQueue _contactReportsGenerationQueue;
        private void ProcessContactReportsGenerationQueue()
        {
            if (_contactReportsGenerationQueue == null)
                _contactReportsGenerationQueue = new ContactReportGenerationQueue();

            var queueProcessor = _contactReportsGenerationQueue.Processor;
            if (!queueProcessor.IsRunning)
            {
                //  Wait 1ms between each processing execution
                //  If the queue is empty wait 10sec until next execution
                //  If an error ocurred during processing wait 30sec until next execution
                queueProcessor.Start(1, 10000, 30000);
            }
        }

        //  Initial approach for processing the "ContactReportGenerationQueue"
        //private ContactReportGenerationQueue _contactReportsGenerationQueue;
        //private void ProcessContactReportsGenerationQueue()
        //{
        //    if (_contactReportsGenerationQueue == null)
        //        _contactReportsGenerationQueue = new ContactReportGenerationQueue();

        //    DequeueReceipt receipt;
        //    var contactReport = _contactReportsGenerationQueue.BeginDequeue(out receipt);
        //    if (contactReport == null)
        //        return;

        //    contactReport.Generate();

        //    _contactReportsGenerationQueue.EndDequeue(receipt);
        //}

        public override bool OnStart()
        {
            // Set the maximum number of concurrent connections 
            ServicePointManager.DefaultConnectionLimit = 12;

            // For information on handling configuration changes
            // see the MSDN topic at http://go.microsoft.com/fwlink/?LinkId=166357.

            return base.OnStart();
        }
    }
}
