﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using mana.Web.UI;
using mana.Web.UI.WebControls;

using Library;

using Website.Utils;

namespace Website.Contacts.EmailAddresses
{
    public partial class Manage : PageBase
    {
        const string EMAILADDRESSES_GRID_ITEMS_KEY = "EmailAddressesGridItems";
        protected List<EmailAddress> EmailAddressesGridItems
        {
            get { return GetFromViewState<List<EmailAddress>>(EMAILADDRESSES_GRID_ITEMS_KEY); }
            set { SetToViewState<List<EmailAddress>>(EMAILADDRESSES_GRID_ITEMS_KEY, value); }
        }

        const string EMAILADDRESSES_SELECTED_ITEM_ID_KEY = "EmailAddressesSelectedItemId";
        protected string EmailAddressesSelectedItemId
        {
            get { return GetFromViewState<string>(EMAILADDRESSES_SELECTED_ITEM_ID_KEY); }
            set { SetToViewState<string>(EMAILADDRESSES_SELECTED_ITEM_ID_KEY, value); }
        }

        const string CONTACT_KEY = "Contact";
        protected Contact Contact
        {
            get { return GetFromViewState<Contact>(CONTACT_KEY); }
            set { SetToViewState<Contact>(CONTACT_KEY, value); }
        }

        protected EmailAddress EmailAddressesSelectedItem
        {
            get
            {
                if (string.IsNullOrWhiteSpace(EmailAddressesSelectedItemId))
                    return null;

                return EmailAddressesGridItems.FindById(Contact.ContactId, EmailAddressesSelectedItemId);
            }
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            ClearMessage(lblEmailAddressesMessage);

            try
            {
                if (!IsPostBack)
                {
                    LoadContact();
                    ShowContactData();

                    LoadEmailAddressesTypes();
                }
            }
            catch (Exception ex)
            {
                ShowMessage(lblEmailAddressesMessage, ex.Message);
            }
        }

        protected void btnNewEmailAddressAccept_Click(object sender, EventArgs e)
        {
            ClearMessage(lblNewEmailAddressMessage);

            try
            {
                AddEmailAddress();

                UpdateEmailAddressesGrid();
                ClearControls();
            }
            catch (Exception ex)
            {
                ShowMessage(lblNewEmailAddressMessage, ex.Message);
            }
        }

        protected void btnEditEmailAddressSave_Click(object sender, EventArgs e)
        {
            ClearMessage(lblEditEmailAddressMessage);

            try
            {
                UpdateEmailAddress();

                UpdateEmailAddressesGrid();
                ClearControls();
            }
            catch (Exception ex)
            {
                ShowMessage(lblEditEmailAddressMessage, ex.Message);
            }
        }

        private void LoadContact()
        {
            string cid = Request.QueryString["cid"];
            if (string.IsNullOrWhiteSpace(cid))
                Response.Redirect("/Contacts/Manage.aspx");

            var contact = Library.Contact.Get(cid);
            if (contact == null)
                Response.Redirect("/Contacts/Manage.aspx");

            Contact = contact;
        }

        private void ShowContactData()
        {
            var contact = Contact;

            //  show contact
            lblContactName.Text = contact.Name;

            //  show email addresses
            contact.LoadEmails();
            EmailAddressesGridItems = contact.Emails.ToList();
            UpdateEmailAddressesGrid();
        }

        private void LoadEmailAddressesTypes()
        {
            ddlNewEmailAddressType.LoadEmailAddressTypes();
            ddlEditEmailAddressType.LoadEmailAddressTypes();
        }

        private void UpdateEmailAddressesGrid()
        {
            grdEmailAddresses.DataBind();
        }

        private void ClearControls()
        {
            txtNewEmailAddressAddress.Text = ddlNewEmailAddressType.Text = lblNewEmailAddressMessage.Text = string.Empty;
            txtEditEmailAddressAddress.Text = ddlEditEmailAddressType.Text = lblEditEmailAddressMessage.Text = string.Empty;
        }

        protected void grdEmailAddresses_DataBinding(object sender, EventArgs e)
        {
            //	The "DataSource" assignation MUST be done in this event, so that it's always performed
            //	It also prevents "Button's Event validation error"

            try
            {
                grdEmailAddresses.DataSource = EmailAddressesGridItems;
                grdEmailAddresses.ManageDataBindingEvent<EmailAddress>();
            }
            catch (Exception ex)
            {
                ShowMessage(lblEmailAddressesMessage, ex.Message);
            }
        }

        protected void grdEmailAddresses_PageIndexChanging(object sender, GridViewPageEventArgs e)
        {
            try
            {
                grdEmailAddresses.ManagePageIndexChangingEvent(e);
            }
            catch (Exception ex)
            {
                ShowMessage(lblEmailAddressesMessage, ex.Message);
            }
        }

        protected void grdEmailAddresses_Sorting(object sender, GridViewSortEventArgs e)
        {
            try
            {
                grdEmailAddresses.ManageSortingEvent(e);
            }
            catch (Exception ex)
            {
                ShowMessage(lblEmailAddressesMessage, ex.Message);
            }
        }

        protected void grdEmailAddresses_RowCommand(object sender, GridViewCommandEventArgs e)
        {
            ClearMessage(lblEmailAddressesMessage);
            ClearControls();

            try
            {
                if (e.CommandName != "EditItem" && e.CommandName != "DeleteItem")
                    return;

                string selectedItemId = Convert.ToString(e.CommandArgument);
                EmailAddressesSelectedItemId = selectedItemId;

                var selectedItem = EmailAddressesSelectedItem;
                if (selectedItem == null)
                    throw new Exception("No item found with the specified id.");

                switch (e.CommandName)
                {
                    case "EditItem":
                        txtEditEmailAddressAddress.Text = selectedItem.Address;
                        ddlEditEmailAddressType.Text = Convert.ToString((int)selectedItem.Type);

                        break;
                    case "DeleteItem":
                        DeleteEmailAddress(selectedItem);
                        UpdateEmailAddressesGrid();

                        break;
                    default:
                        throw new Exception("Invalid grid operation.");
                }
            }
            catch (Exception ex)
            {
                ShowMessage(lblEmailAddressesMessage, ex.Message);
            }
        }

        private void AddEmailAddress()
        {
            //	add item to DB
            var newEmailAddress = Contact.AddEmail(txtNewEmailAddressAddress.Text, (EmailAddressType)int.Parse(ddlNewEmailAddressType.SelectedValue));

            //	add item to Property
            EmailAddressesGridItems.Add(newEmailAddress);
        }

        private void UpdateEmailAddress()
        {
            var selectedItem = EmailAddressesSelectedItem;
            if (selectedItem == null)
                return;

            //	edit item in Property
            selectedItem.Address = txtEditEmailAddressAddress.Text;
            selectedItem.Type = (EmailAddressType)int.Parse(ddlEditEmailAddressType.SelectedValue);

            //	edit item in DB
            selectedItem.Update();
        }

        private void DeleteEmailAddress(EmailAddress item)
        {
            if (item == null)
                return;

            //	delete from DB
            Contact.DeleteEmail(item);

            //	delete from Property
            EmailAddressesGridItems.Remove(item);
        }
    }
}