﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using mana.Web.UI;
using mana.Web.UI.WebControls;

using Library;

namespace Website.Contacts
{
    public partial class Manage : PageBase
    {
        const string CONTACTS_GRID_ITEMS_KEY = "ContactsGridItems";
        protected List<Contact> ContactsGridItems
        {
            get { return GetFromViewState<List<Contact>>(CONTACTS_GRID_ITEMS_KEY); }
            set { SetToViewState<List<Contact>>(CONTACTS_GRID_ITEMS_KEY, value); }
        }

        const string CONTACTS_SELECTED_ITEM_ID_KEY = "ContactsSelectedItemId";
        protected string ContactsSelectedItemId
        {
            get { return GetFromViewState<string>(CONTACTS_SELECTED_ITEM_ID_KEY); }
            set { SetToViewState<string>(CONTACTS_SELECTED_ITEM_ID_KEY, value); }
        }

        protected Contact ContactsSelectedItem
        {
            get
            {
                if (string.IsNullOrWhiteSpace(ContactsSelectedItemId))
                    return null;

                return ContactsGridItems.FindById(ContactsSelectedItemId);
            }
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            ClearMessage(lblContactsMessage);

            try
            {
                if (!IsPostBack)
                {
                    ContactsGridItems = Library.Contact.GetAll();
                    UpdateContactsGrid();
                }
            }
            catch (Exception ex)
            {
                ShowMessage(lblContactsMessage, ex.Message);
            }
        }

        protected void btnNewContactAccept_Click(object sender, EventArgs e)
        {
            ClearMessage(lblNewContactMessage);

            try
            {
                AddContact();

                UpdateContactsGrid();
                ClearControls();
            }
            catch (Exception ex)
            {
                ShowMessage(lblNewContactMessage, ex.Message);
            }
        }

        protected void btnEditContactSave_Click(object sender, EventArgs e)
        {
            ClearMessage(lblEditContactMessage);

            try
            {
                UpdateContact();

                UpdateContactsGrid();
                ClearControls();
            }
            catch (Exception ex)
            {
                ShowMessage(lblEditContactMessage, ex.Message);
            }
        }

        private void UpdateContactsGrid()
        {
            grdContacts.DataBind();
        }

        private void ClearControls()
        {
            txtNewContactName.Text = lblNewContactMessage.Text = string.Empty;
            txtEditContactName.Text = lblEditContactCreationDate.Text = lblEditContactMessage.Text = string.Empty;
        }

        protected void grdContacts_DataBinding(object sender, EventArgs e)
        {
            //	The "DataSource" assignation MUST be done in this event, so that it's always performed
            //	It also prevents "Button's Event validation error"
            grdContacts.DataSource = ContactsGridItems;
            grdContacts.ManageDataBindingEvent<Contact>();
        }

        protected void grdContacts_PageIndexChanging(object sender, GridViewPageEventArgs e)
        {
            try
            {
                grdContacts.ManagePageIndexChangingEvent(e);
            }
            catch (Exception ex)
            {
                lblContactsMessage.Text = ex.Message;
            }
        }

        protected void grdContacts_Sorting(object sender, GridViewSortEventArgs e)
        {
            try
            {
                grdContacts.ManageSortingEvent(e);
            }
            catch (Exception ex)
            {
                lblContactsMessage.Text = ex.Message;
            }
        }

        protected void grdContacts_RowCommand(object sender, GridViewCommandEventArgs e)
        {
            ClearMessage(lblContactsMessage);
            ClearControls();

            try
            {
                if (e.CommandName != "EditItem" && e.CommandName != "DeleteItem")
                    return;

                string selectedItemId = Convert.ToString(e.CommandArgument);
                ContactsSelectedItemId = selectedItemId;

                var selectedItem = ContactsSelectedItem;
                if (selectedItem == null)
                    throw new Exception("No contact found with the specified id.");

                switch (e.CommandName)
                {
                    case "EditItem":
                        txtEditContactName.Text = selectedItem.Name;
                        lblEditContactCreationDate.Text = selectedItem.CreationDate.ToString("yyyy/MM/dd");

                        break;
                    case "DeleteItem":
                        DeleteContact(selectedItem);
                        UpdateContactsGrid();

                        break;
                    default:
                        throw new Exception("Invalid grid operation.");
                }
            }
            catch (Exception ex)
            {
                ShowMessage(lblContactsMessage, ex.Message);
            }
        }

        private void AddContact()
        {
            //	add item to DB
            var newContact = Library.Contact.Add(txtNewContactName.Text);

            //	add item to Property
            ContactsGridItems.Add(newContact);
        }

        private void UpdateContact()
        {
            var selectedItem = ContactsSelectedItem;
            if (selectedItem == null)
                return;

            //	edit item in Property
            selectedItem.Name = txtEditContactName.Text;

            //	edit item in DB
            selectedItem.Update();
        }

        private void DeleteContact(Contact item)
        {
            if (item == null)
                return;

            //	delete from DB
            Library.Contact.Delete(item);

            //	delete from Property
            ContactsGridItems.Remove(item);
        }
    }
}