﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

using mana.Web.UI;
using mana.Web.UI.WebControls;

using Library;

using Website.Utils;

namespace Website.Contacts.PhoneNumbers
{
    public partial class Manage : PageBase
    {
        const string PHONENUMBERS_GRID_ITEMS_KEY = "PhoneNumbersGridItems";
        protected List<PhoneNumber> PhoneNumbersGridItems
        {
            get { return GetFromViewState<List<PhoneNumber>>(PHONENUMBERS_GRID_ITEMS_KEY); }
            set { SetToViewState<List<PhoneNumber>>(PHONENUMBERS_GRID_ITEMS_KEY, value); }
        }

        const string PHONENUMBERS_SELECTED_ITEM_ID_KEY = "PhoneNumbersSelectedItemId";
        protected string PhoneNumbersSelectedItemId
        {
            get { return GetFromViewState<string>(PHONENUMBERS_SELECTED_ITEM_ID_KEY); }
            set { SetToViewState<string>(PHONENUMBERS_SELECTED_ITEM_ID_KEY, value); }
        }

        const string CONTACT_KEY = "Contact";
        protected Contact Contact
        {
            get { return GetFromViewState<Contact>(CONTACT_KEY); }
            set { SetToViewState<Contact>(CONTACT_KEY, value); }
        }

        protected PhoneNumber PhoneNumbersSelectedItem
        {
            get
            {
                if (string.IsNullOrWhiteSpace(PhoneNumbersSelectedItemId))
                    return null;

                return PhoneNumbersGridItems.FindById(Contact.ContactId, PhoneNumbersSelectedItemId);
            }
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            ClearMessage(lblPhoneNumbersMessage);

            try
            {
                if (!IsPostBack)
                {
                    LoadContact();
                    ShowContactData();

                    LoadPhoneNumberTypes();
                }
            }
            catch (Exception ex)
            {
                ShowMessage(lblPhoneNumbersMessage, ex.Message);
            }
        }

        protected void btnNewPhoneNumberAccept_Click(object sender, EventArgs e)
        {
            ClearMessage(lblNewPhoneNumberMessage);

            try
            {
                AddPhoneNumber();

                UpdatePhoneNumbersGrid();
                ClearControls();
            }
            catch (Exception ex)
            {
                ShowMessage(lblNewPhoneNumberMessage, ex.Message);
            }
        }

        protected void btnEditPhoneNumberSave_Click(object sender, EventArgs e)
        {
            ClearMessage(lblEditPhoneNumberMessage);

            try
            {
                UpdatePhoneNumber();

                UpdatePhoneNumbersGrid();
                ClearControls();
            }
            catch (Exception ex)
            {
                ShowMessage(lblEditPhoneNumberMessage, ex.Message);
            }
        }

        private void LoadContact()
        {
            string cid = Request.QueryString["cid"];
            if (string.IsNullOrWhiteSpace(cid))
                Response.Redirect("/Contacts/Manage.aspx");

            var contact = Library.Contact.Get(cid);
            if (contact == null)
                Response.Redirect("/Contacts/Manage.aspx");

            Contact = contact;
        }

        private void ShowContactData()
        {
            var contact = Contact;

            //  show contact
            lblContactName.Text = contact.Name;

            //  show Phones
            contact.LoadPhones();
            PhoneNumbersGridItems = contact.Phones.ToList();
            UpdatePhoneNumbersGrid();
        }

        private void LoadPhoneNumberTypes()
        {
            ddlNewPhoneNumberType.LoadPhoneNumberTypes();
            ddlEditPhoneNumberType.LoadPhoneNumberTypes();
        }

        private void UpdatePhoneNumbersGrid()
        {
            grdPhoneNumbers.DataBind();
        }

        private void ClearControls()
        {
            txtNewPhoneNumberNumber.Text = ddlNewPhoneNumberType.SelectedValue = lblNewPhoneNumberMessage.Text = string.Empty;
            txtEditPhoneNumberNumber.Text = ddlEditPhoneNumberType.SelectedValue = lblEditPhoneNumberMessage.Text = string.Empty;
        }

        protected void grdPhoneNumbers_DataBinding(object sender, EventArgs e)
        {
            //	The "DataSource" assignation MUST be done in this event, so that it's always performed
            //	It also prevents "Button's Event validation error"

            try
            {
                grdPhoneNumbers.DataSource = PhoneNumbersGridItems;
                grdPhoneNumbers.ManageDataBindingEvent<PhoneNumber>();
            }
            catch (Exception ex)
            {
                ShowMessage(lblPhoneNumbersMessage, ex.Message);
            }
        }

        protected void grdPhoneNumbers_PageIndexChanging(object sender, GridViewPageEventArgs e)
        {
            try
            {
                grdPhoneNumbers.ManagePageIndexChangingEvent(e);
            }
            catch (Exception ex)
            {
                ShowMessage(lblPhoneNumbersMessage, ex.Message);
            }
        }

        protected void grdPhoneNumbers_Sorting(object sender, GridViewSortEventArgs e)
        {
            try
            {
                grdPhoneNumbers.ManageSortingEvent(e);
            }
            catch (Exception ex)
            {
                ShowMessage(lblPhoneNumbersMessage, ex.Message);
            }
        }

        protected void grdPhoneNumbers_RowCommand(object sender, GridViewCommandEventArgs e)
        {
            ClearMessage(lblPhoneNumbersMessage);
            ClearControls();

            try
            {
                if (e.CommandName != "EditItem" && e.CommandName != "DeleteItem")
                    return;

                string selectedItemId = Convert.ToString(e.CommandArgument);
                PhoneNumbersSelectedItemId = selectedItemId;

                var selectedItem = PhoneNumbersSelectedItem;
                if (selectedItem == null)
                    throw new Exception("No item found with the specified id.");

                switch (e.CommandName)
                {
                    case "EditItem":
                        txtEditPhoneNumberNumber.Text = selectedItem.Number;
                        ddlEditPhoneNumberType.SelectedValue = Convert.ToString((int)selectedItem.Type);

                        break;
                    case "DeleteItem":
                        DeletePhoneNumber(selectedItem);
                        UpdatePhoneNumbersGrid();

                        break;
                    default:
                        throw new Exception("Invalid grid operation.");
                }
            }
            catch (Exception ex)
            {
                ShowMessage(lblPhoneNumbersMessage, ex.Message);
            }
        }

        private void AddPhoneNumber()
        {
            //	add item to DB
            var newPhoneNumber = Contact.AddPhone(txtNewPhoneNumberNumber.Text, (PhoneNumberType)int.Parse(ddlNewPhoneNumberType.SelectedValue));

            //	add item to Property
            PhoneNumbersGridItems.Add(newPhoneNumber);
        }

        private void UpdatePhoneNumber()
        {
            var selectedItem = PhoneNumbersSelectedItem;
            if (selectedItem == null)
                return;

            //	edit item in Property
            selectedItem.Number = txtEditPhoneNumberNumber.Text;
            selectedItem.Type = (PhoneNumberType)int.Parse(ddlEditPhoneNumberType.SelectedValue);

            //	edit item in DB
            selectedItem.Update();
        }

        private void DeletePhoneNumber(PhoneNumber item)
        {
            if (item == null)
                return;

            //	delete from DB
            Contact.DeletePhone(item);

            //	delete from Property
            PhoneNumbersGridItems.Remove(item);
        }
    }
}