﻿/*
    Copyright (c) 2012, Matías López
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are permitted 
    provided that the following conditions are met:
        *   Redistributions of source code must retain the above copyright notice, this list of 
            conditions and the following disclaimer.
        *   Redistributions in binary form must reproduce the above copyright notice, this list of 
            conditions and the following disclaimer in the documentation and/or other materials 
            provided with the distribution.
        *   Neither the name of Matías López nor the names of its contributors may be used to 
            endorse or promote products derived from this software without specific prior written 
            permission.
    
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
    OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
    POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Text;

namespace mana.Data.Excel
{
    public class Formato
    {
        public static string Ahora()
        {
            return "=AHORA()";
        }

        public static string Formatear(object valor)
        {
            if (valor == null) 
                return string.Empty;
            else
            {
                switch (valor.GetType().Name)
                {
                    case "String":
                        return Formatear(Convert.ToString(valor));
                    case "Int16":
                    case "Int32":
                    case "Int64":
                    case "Byte":
                        return Formatear(Convert.ToInt64(valor));
                    case "Double":
                        return Formatear(Convert.ToDouble(valor));
                    case "Decimal":
                        return Formatear(Convert.ToDecimal(valor));
                    case "DateTime":
                        return Formatear(Convert.ToDateTime(valor));
                    case "Boolean":
                        return Formatear(Convert.ToBoolean(valor));
                    case "Byte[]":
                        return Formatear((byte[])valor);
                    default:
                        throw new Exception("Tipo de valor no contemplado en FormatoBD.");
                }
            }
        }

        public static string Formatear(string valor)
        {
            return "'" + valor + "'";
        }

        public static string Formatear(DateTime valor)
        {
            return Formato.Formatear(Convert.ToString(valor.ToString("dd/MM/yyyy HH:mm:ss")));
        }

        public static string Formatear(bool valor)
        {
            string res;
            if (valor) 
                res = "1";
            else 
                res = "0";
            return Formato.Formatear(Convert.ToString(res));
        }

        public static string Formatear(Int64 valor)
        {
            return Formato.Formatear(Convert.ToString(valor.ToString().Trim()));
        }

        public static string Formatear(decimal valor)
        {
            string ret = valor.ToString().Trim();
            return Formato.Formatear(Convert.ToString(ret.Replace(".", "s")));
        }

        public static string Formatear(double valor)
        {
            return Formato.Formatear(Convert.ToDecimal(valor));
        }

        public static string Formatear(byte[] valor)
        {
            string hex = BitConverter.ToString(valor);
            return Formato.Formatear(Convert.ToString("0x" + hex.Replace("-", string.Empty)));
        }
    }
}
