﻿/*
    Copyright (c) 2012, Matías López
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are permitted 
    provided that the following conditions are met:
        *   Redistributions of source code must retain the above copyright notice, this list of 
            conditions and the following disclaimer.
        *   Redistributions in binary form must reproduce the above copyright notice, this list of 
            conditions and the following disclaimer in the documentation and/or other materials 
            provided with the distribution.
        *   Neither the name of Matías López nor the names of its contributors may be used to 
            endorse or promote products derived from this software without specific prior written 
            permission.
    
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
    OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
    POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;

namespace mana.Data.SQL
{
    public class Materializer<T>
    {
        protected Materializer()
        { }

        #region Public

        public T ReadInstance(string cmd, Transaccion tr)
        {
            return ReadInstance(cmd, null, tr);
        }

        public T ReadInstance(string cmd, IEnumerable<SqlParameter> parameters, Transaccion tr)
        {
            var reader = tr.ExecuteReader(cmd, parameters);
            T res = default(T);
            try
            {
                if (reader.Read())
                    res = MaterializeFromReader(reader);
            }
            finally
            {
                reader.Close();
            }
            return res;
        }

        public List<T> ReadCollection(string cmd, Transaccion tr)
        {
            return ReadCollection(cmd, null, tr);
        }

        public List<T> ReadCollection(string cmd, IEnumerable<SqlParameter> parameters, Transaccion tr)
        {
            var reader = tr.ExecuteReader(cmd, parameters);
            var res = new List<T>();
            try
            {
                while (reader.Read())
                {
                    res.Add(MaterializeFromReader(reader));
                }
            }
            finally
            {
                reader.Close();
            }
            return res;
        }

        #endregion Public

        #region Protected

        protected virtual T OnInstantiateObject(DataReader reader)
        {
            throw new NotImplementedException();
        }

        #endregion Protected

        #region Private

        private T MaterializeFromReader(DataReader reader)
        {
            if (reader == null)
                return default(T);

            return OnInstantiateObject(reader);
        }

        #endregion Private
    }
}
