﻿/*
    Copyright (c) 2012, Matías López
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are permitted 
    provided that the following conditions are met:
        *   Redistributions of source code must retain the above copyright notice, this list of 
            conditions and the following disclaimer.
        *   Redistributions in binary form must reproduce the above copyright notice, this list of 
            conditions and the following disclaimer in the documentation and/or other materials 
            provided with the distribution.
        *   Neither the name of Matías López nor the names of its contributors may be used to 
            endorse or promote products derived from this software without specific prior written 
            permission.
    
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
    OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
    POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace mana.Data.SQL.Schema
{
    public static class SqlTableSchemaRestrictionsTableType
    {
        public const string View = "VIEW";
        public const string BaseTable = "BASE TABLE";
    }

    public class SqlSchemaRestrictionsDatabases : ISqlSchemaRestrictions
    {
        /*
        Name	@Name	Name	1
        */

        public string Name { get; set; }

        public string[] Value
        {
            get { return new string[1] { Name }; }
        }
    }

    public class SqlSchemaRestrictionsTables : ISqlSchemaRestrictions
    {
        /*
        Catalog	    @Catalog	TABLE_CATALOG	1
        Owner	    @Owner	    TABLE_SCHEMA	2
        Table	    @Name	    TABLE_NAME	3
        TableType	@TableType	TABLE_TYPE	4
        */

        public string Catalog { get; set; }
        public string Owner { get; set; }
        public string Table { get; set; }
        public string TableType { get; set; }

        public string[] Value
        {
            get
            {
                return new string[4] { Catalog, Owner, Table, TableType };
            }
        }
    }

    public class SqlSchemaRestrictionsColumns : ISqlSchemaRestrictions
    {
        /*
        Catalog	@Catalog	TABLE_CATALOG
        Owner	@Owner	    TABLE_SCHEMA
        Table	@Table	    TABLE_NAME
        Column	@Column	    COLUMN_NAME
        */

        public string Catalog { get; set; }
        public string Owner { get; set; }
        public string TableName { get; set; }
        public string ColumnName { get; set; }

        public string[] Value
        {
            get
            {
                return new string[4] { Catalog, Owner, TableName, ColumnName };
            }
        }
    }

    public interface ISqlSchemaRestrictions
    {
        string[] Value { get; }
    }
}
