﻿/*
    Copyright (c) 2012, Matías López
    All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are permitted 
    provided that the following conditions are met:
        *   Redistributions of source code must retain the above copyright notice, this list of 
            conditions and the following disclaimer.
        *   Redistributions in binary form must reproduce the above copyright notice, this list of 
            conditions and the following disclaimer in the documentation and/or other materials 
            provided with the distribution.
        *   Neither the name of Matías López nor the names of its contributors may be used to 
            endorse or promote products derived from this software without specific prior written 
            permission.
    
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
    IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY 
    AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
    OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
    POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace mana.Data.SQL.Schema
{
    /*
        table_catalog   String  Catalog of the table.
        table_schema    String  Schema that contains the table.
        table_name      String  Table name.
        table_type      String  Type of table. Can be VIEW or BASE TABLE.
     */
    public class SqlTableSchema
    {
        public SqlTableSchema()
        {
            Columns = new List<SqlColumnSchema>();
        }

        public string Schema { get; set; }
        public string Name { get; set; }

        public string FormattedName
        {
            get { return string.Format("[{0}]", Name); }
        }

        public List<SqlColumnSchema> Columns { get; set; }
        
        public IEnumerable<string> ColumnsFormattedNames
        {
            get { return Columns.Select(c => c.FormattedName); }
        }

        public string ColumnsFormattedNamesString
        {
            get { return string.Join(", ", ColumnsFormattedNames); }
        }

        public override string ToString()
        {
            return FormattedName;
        }
    }
}
